#![feature(round_char_boundary)]
use crate::parser::document;
use tracing::Level;
use tracing_subscriber::fmt::format::FmtSpan;

mod parser;

const TEST_DOC: &'static str = include_str!("../toy_language.txt");

fn main() -> Result<(), Box<dyn std::error::Error>> {
    let format = tracing_subscriber::fmt::format()
        .with_file(true)
        .with_line_number(true)
        .with_thread_ids(false)
        .with_target(false);
    let subscriber = tracing_subscriber::fmt()
        .event_format(format)
        .with_max_level(Level::TRACE)
        .with_span_events(FmtSpan::ENTER | FmtSpan::EXIT)
        .finish();
    tracing::subscriber::set_global_default(subscriber)?;
    let parsed = document(TEST_DOC);
    println!("{}\n\n\n", TEST_DOC);
    println!("{:#?}", parsed);
    Ok(())
}
